@@ -2,14 +2,14 @@ package Panda::Config::Perl;
 use 5.012;
 use Panda::Lib(); # XS code needs xs::lib::merge
 
-our $VERSION = '1.1.5';
+our $VERSION = '1.1.7';
 
 require Panda::XSLoader;
 Panda::XSLoader::load();
 
 sub process {
     my ($class, $file, $initial_cfg) = @_;
-    
+
     my ($mstash, $nsstash);
     {
         no strict 'refs';
@@ -17,16 +17,22 @@ sub process {
         delete $mstash->{'NS::'};
         $nsstash = \%{"NS::"};
     }
-    
+
     _apply_initial_cfg('', Panda::Lib::fclone($initial_cfg)) if $initial_cfg;
     _process_file($file);
-    
+
     my $ret = {};
-    _get_config($ret, $nsstash, '');
-    
+
+    my $cfg = {};
+    if(defined $nsstash->{'__CONFIG__'}){
+        $cfg = ${$nsstash->{'__CONFIG__'}};
+        delete $nsstash->{'__CONFIG__'};
+    }
+    _get_config($ret, $nsstash, $cfg, '');
+
     # remove garbage we've created
     delete $mstash->{'NS::'};
-    
+
     return $ret;
 }
 
@@ -51,14 +57,14 @@ sub _process_file {
         $content = <$fh>;
     }
     close $fh;
-    
+
     my $curdir = '';
     if ((my $pos = rindex($file, '/')) >= 0) {
         $curdir = substr($file, 0, $pos);
     }
-    
+
     $content =~ s/^[^\S\r\n]*#(namespace|namespace-abs|include)(?:[^\S\r\n]+(.+))?$/_process_directive($curdir, $ns, $1, $2)/gme;
-    
+
     my $pkg = $ns ? "NS::$ns" : "NS";
     $content = "package $pkg; sub { $content;\n }";
     my $ok;
@@ -78,7 +84,7 @@ sub _process_file {
             _content_linno($content).
             "==================================================";
     }
-    
+
     return;
 }
 
@@ -113,8 +119,11 @@ sub _INCLUDE {
 }
 
 sub _get_config {
-    my ($dest, $stash, $ns) = @_;
+    my ($dest, $stash, $config, $ns) = @_;
     my @ns_list;
+
+    my $assign_proc;
+    $assign_proc = $config->{assign_proc} if defined $config->{assign_proc};
     foreach my $key (keys %$stash) {
         next if $key eq 'BEGIN' or $key eq 'DESTROY' or $key eq 'AUTOLOAD' or index($key, '__ANON__') == 0;
         if (substr($key, -2, 2) eq '::') {
@@ -123,18 +132,23 @@ sub _get_config {
         }
         my $glob = $stash->{$key} or next;
         next if !defined $$glob and defined *$glob{CODE};
-        $dest->{$key} = $$glob;
+        if(defined $assign_proc){
+            $dest->{$key} = undef;
+            $assign_proc->($dest->{$key}, $$glob);
+        } else {
+            $dest->{$key} = $$glob;
+        }
     }
-    
+
     foreach my $subns (@ns_list) {
         my $substash = \%{$stash->{$subns}};
         substr($subns, -2, 2, '');
-        my $subns_full = $ns ? "${ns}::$subns}" : $subns;
+        my $subns_full = $ns ? "${ns}::$subns" : $subns;
         if (exists $dest->{$subns}) {
             die "Panda::Config::Perl: conflict between variable '$subns' in namespace '$ns' and a namespace '$subns_full'. ".
                 "You shouldn't have variables that overlap with namespaces as they would merge into the same hash.\n";
         }
-        _get_config($dest->{$subns} = {}, $substash, $subns_full);
+        _get_config($dest->{$subns} = {}, $substash, $config, $subns_full);
     }
 }
 
